/*
  * File:        lcd.c
  * Auteur:         Pascal Chour
  * date            08/08/2023
  * Version         1.0
  * Gestion afficheur GDC0689
*/

/**
 * \file lcd.c
 * \brief LCD handling functions
 * \author Peter Ivanov, Muneem Shahriar, Pascal Chour
 */
// Contains code from Muneem Shahriar
#include <msp430f449.h>
#include <stdint.h>
#include "lcd.h"


//#define __DEBUG__   1

char *LCD = LCDMEM;

void LCD_init ()
{
    // Initialize LCD
    LCD_clear ();

    TACTL = TASSEL0 | TACLR;                // set timer_A
    LCDCTL = LCDON + LCD4MUX + LCDSG0_7;    // STK LCD 4mux LCD, S0-S39
    //BTCTL = BT_fLCD_DIV64;                // STK LCD freq ACLK/64
    BTCTL = BT_fLCD_DIV128;                 // set LCD frame freq = ACLK
    P5SEL = 0xFC;                           // set Rxx and COM pins for LCD
}

/*
 * Efface l'afficheur
 */
void LCD_clear ()
{
    uint8_t i;
    for (i = 0; i < 20; i++) 
    {
        LCD[i] = 0x00; //clear Sxx
    }
}

/*
 * Allume tous les segments de lafficheur
 */
void LCD_all ()
{
    uint8_t i;
    unsigned int j;
    for (i = 0; i < 20; i++)
    {
      for (j = 0; j <= 255; j++)
      {
        LCD[i] = j; // turn on all LCD segments
      }
    }
}



/* pas de contrle de dbordement !
 * length : longueur du texte  afficher avec les points dcimaux s'il y a lieu
 *
 */
void LCD_write (const char *buf, uint8_t length)
{
   uint8_t i;
   int8_t pos;

   pos = 5;

   for (i = 0; i < length && pos > 0; i++) // display str
   {
       if (pos > 0) {
           LCD_writepoint (pos, FALSE);
       }
       if (buf[i] == '.')
       {
           if (pos < 5)
           {
               LCD_writepoint (pos+1, TRUE);
           }
       } else {
           LCD_writeafficheurbas (pos, buf[i]);  // displays each letter in the str
           pos = pos-1;
       }
   }
}


/*
Affiche les segments de la batterie 
level : 0  3 en fonction du niveau 
*/
void LCD_showBatteryLevel (unsigned int level)
{
  LCD[6] &= 0xF0; // batterie vide
  LCD[2] |= 0x08; // corps de batterie
  switch (level)
    {
        case 1: LCD[6] |= 1 ;        break;  // batterie presque dcharge
        case 2: LCD[6] |= 3 ;        break;  // batterie pas mal dcharge
        case 3: LCD[6] |= 7 ;        break;  // batterie un peu dcharge
        case 4 :LCD[6] |= 15 ;       break;  // batterie charge
    }
}



/* affichage ou extinction du "cl" */
void LCD_cle(bool_t on)
{
  if (on) {
      LCD[2] |= 0x04;
  } else {
      LCD[2] &= 0xFB;
  }
}

/*
Affiche le bargraph  (1  3 barremax et force l'affichage de l'antenne, sinon, a n'a pas de sens

*/
void LCD_bargraph (unsigned int Niveau)
{
  LCD[6] &= 0x1F; // Efface bargraph sauf antenne
  switch (Niveau) {
  case 0: LCD[6] |= 0x10; break; // Antenne + 0 barre
  case 1: LCD[6] |= 0x30; break; // Antenne + 1 barre
  case 2: LCD[6] |= 0x70; break; // Antenne + 2 barre
  case 3: LCD[6] |= 0xF0; break; // Antenne + 3 barre
  }
}

/* affichage ou effacement de l'antenne. Le Bargraph est effac */
void LCD_antenne(bool_t on)
{
  if (on) {
    LCD[6] &= 0x1F; // Efface bargraph sauf antenne
    LCD[6] |= 0x10; // Alume antenne
  } else {
    LCD[6] &= 0x0F; // Efface Bargraph et antenne
  }
}

/* Gestion du clignotement de l'afficheur */
void LCD_blink(bool_t on)
{
  if (on) {
    LCDCTL |= LCDSON;
  } else {
    LCDCTL &= ~LCDSON;
  }
}

void LCD_test ()
{
unsigned char j;
unsigned char k;
unsigned char l;
    for (j=0;j<20;j++)
    {
        l = 1;
        for (k=0;k<=7;k++)
        {
            //LCD[j] = (~l);
            LCD[j] = l;
            l = l << 1;
            LCD_clear();
        }
    }
}


/* affichage ou extinction d'un point
 * Les positions possible sont 2  6
*/

void LCD_writepoint(unsigned int position, bool_t on)
{
    unsigned char point;
    point = 0x80;
    switch (position)
    {
    case 2: position = 13; break;
    case 3: position = 15; break;
    case 4: position = 16; break;
    case 5: position = 17; break;
    default: position = 8;  point = 0x08;
    }
    if (on) {
        LCD[position] |= point;
    } else {
        LCD[position] &= ~point;
    }
}

/*
 *  crit les deux points
 *  0 = afficheur haut (COL1), 1 = afficheur bas (COL2), 2 = afficheur bas (COL3)
*/

void LCD_deuxpoints (unsigned int col, bool_t on)
{
    switch (col)
    {
        case 0: col = 16; break;
        case 1: col = 1; break;
        case 2: col = 2; break;
        default: return;
    }
    if (on) {
        LCD[2] |= col;
    } else {
        LCD[2] &= ~col;
    }
}

/*
 *  crit un caractre (s'il existe)  une position donne sur l'afficheur du bas
 *  Position : position de 0  5, 0 est la positon la plus  droite
 *  letter : lettre  afficher
 */


void LCD_writeafficheurbas (unsigned int position, char letter)
{
    unsigned char i;
    unsigned char j;
    if (letter >= 'a' && letter <= 'z')
    {
        letter &= 0xDF; // majuscule
    }
    // comme l'afficheur est cbl n'importe comment, il faut tout grer  la main
    position = position+1;
    switch (position)
    {
    case 1:
    case 2: position = position+6; break;
    case 3: position = 13; break;
    case 4: position = 15; break;
    case 5: position = 16; break;
    case 6: position = 17; break;
    default: position = 7;
    }

    switch (position)
    {
    case 7:
    case 8:
        i = LCD[position] & 0x08;
        switch(letter)
        {
            case '0': LCD[position] = 0xD7; break;
            case '1': LCD[position] = 0x06; break;
            case '2': LCD[position] = 0xE3; break;
            case '3': LCD[position] = 0xA7; break;
            case '4': LCD[position] = 0x36; break;
            case '5': LCD[position] = 0xB5; break;
            case '6': LCD[position] = 0xF5; break;
            case '7': LCD[position] = 0x07; break;
            case '8': LCD[position] = 0xF7; break;
            case '9': LCD[position] = 0xB7; break;
            case 'A': LCD[position] = 0x77; break;
            case 'B': LCD[position] = 0xF4; break;
            case 'C': LCD[position] = 0xE0; break;
            case 'D': LCD[position] = 0xE6; break;
            case 'E': LCD[position] = 0xF1; break;
            case 'F': LCD[position] = 0x71; break;
            case 'G': LCD[position] = 0xD5; break;
            case 'H': LCD[position] = 0x74; break;
            case 'I': LCD[position] = 0x04; break;
            case 'J': LCD[position] = 0xC6; break;
            case 'L': LCD[position] = 0xB0; break;
            case 'N': LCD[position] = 0x64; break;
            case 'O': LCD[position] = 0xE4; break;
            case 'P': LCD[position] = 0x73; break;
            case 'Q': LCD[position] = 0xB7; break;
            case 'R': LCD[position] = 0x60; break;
            case 'S': LCD[position] = 0xB5; break;
            case 'T': LCD[position] = 0xF0; break;
            case 'U': LCD[position] = 0xC4; break;
            case 'Y': LCD[position] = 0x36; break;
            default:LCD[position] = 0x00; break; // manque K, M, V, W, X, Z
        }
        LCD[position] |= i;
        break;
        default: // 3  6
            i = LCD[position] & 0x8F;
            j = LCD[position+1] & 0xF0;
            switch(letter)
            {
                case '0': LCD[position] = 0x70; LCD[position+1] = 0x0D;  break;
                case '1': LCD[position] = 0x60; LCD[position+1] = 0x00;  break;
                case '2': LCD[position] = 0x30; LCD[position+1] = 0x0E;  break;
                case '3': LCD[position] = 0x70; LCD[position+1] = 0x0A;  break;
                case '4': LCD[position] = 0x60; LCD[position+1] = 0x03;  break;
                case '5': LCD[position] = 0x50; LCD[position+1] = 0x0B;  break;
                case '6': LCD[position] = 0x50; LCD[position+1] = 0x0F;  break;
                case '7': LCD[position] = 0x70; LCD[position+1] = 0x00;  break;
                case '8': LCD[position] = 0x70; LCD[position+1] = 0x0F;  break;
                case '9': LCD[position] = 0x70; LCD[position+1] = 0x0B;  break;
                case 'A': LCD[position] = 0x70; LCD[position+1] = 0x07;  break;
                case 'B': LCD[position] = 0x40; LCD[position+1] = 0x0F;  break;
                case 'C': LCD[position] = 0x00; LCD[position+1] = 0x0E;  break;
                case 'D': LCD[position] = 0x60; LCD[position+1] = 0x0E;  break;
                case 'E': LCD[position] = 0x10; LCD[position+1] = 0x0F;  break;
                case 'F': LCD[position] = 0x10; LCD[position+1] = 0x07;  break;
                case 'G': LCD[position] = 0x50; LCD[position+1] = 0x0D;  break;
                case 'H': LCD[position] = 0x40; LCD[position+1] = 0x07;  break;
                case 'I': LCD[position] = 0x40; LCD[position+1] = 0x00;  break;
                case 'J': LCD[position] = 0x60; LCD[position+1] = 0x0C;  break;
                case 'L': LCD[position] = 0x00; LCD[position+1] = 0x0D;  break;
                case 'N': LCD[position] = 0x40; LCD[position+1] = 0x06;  break;
                case 'O': LCD[position] = 0x40; LCD[position+1] = 0x0E;  break;
                case 'P': LCD[position] = 0x30; LCD[position+1] = 0x07;  break;
                case 'Q': LCD[position] = 0x70; LCD[position+1] = 0x03;  break;
                case 'R': LCD[position] = 0x00; LCD[position+1] = 0x06;  break;
                case 'S': LCD[position] = 0x50; LCD[position+1] = 0x0B;  break;
                case 'T': LCD[position] = 0x00; LCD[position+1] = 0x0F;  break;
                case 'U': LCD[position] = 0x40; LCD[position+1] = 0x0C;  break;
                case 'Y': LCD[position] = 0x60; LCD[position+1] = 0x03;  break;
                default:  LCD[position] = 0x00; LCD[position+1] = 0x00;  break;
            }
            LCD[position] |= i;
            LCD[position+1] |= j;
    }
}

/*
 * Ecrit 2 digits  une position donne
 * En entre :
 * position = position du caractre le plus  droite
 * Nombre = valeur  crire (0  99). Rsultat imprvisible si Nombre > 99
 *
 */
void LCD_2digitafficheurbas(unsigned int position, int nombre)
{
    int Temp = nombre / 10;
    LCD_writeafficheurbas (position, (nombre-(Temp*10)) | 0x30); // Attention aux optimisations !
    LCD_writeafficheurbas (position+1, Temp | 0x30); // Attention aux optimisations !
}


/*
 * si besoin :
 * Point 3 = (19,0x10), point 2 = (0,0x01), point 1 = (3, 0x10), semicolon = (2,0x10)
 * Comme l'afficheur est cbl n'importe comment, il faut tout grer  la main !
 * Position de 3  0
 * digit = 0..9. Autre chose = effacement
 */
void LCD_writeafficheurhaut (uint8_t position, unsigned char digit)
{
    unsigned char a;
    unsigned char b;
    unsigned char c;
    unsigned char d;
    unsigned char e;
    unsigned char f;
    unsigned char g;
    if (position > 3) {position = 0;}
    switch (position)
    {
    case 2: a = 0x08; b = 0x04; c = 0x02; d = 0x10; e = 0x20; f = 0x80; g = 0x40; break;
    default:a = 0x80; b = 0x40; c = 0x20; d = 0x01; e = 0x02; f = 0x08; g = 0x04;
    }
    switch(digit)
    {
        case '0': digit = a | b | c | d | e | f; break;
        case '1': digit = b | c ; break;
        case '2': digit = a | b | d | e | g; break;
        case '3': digit = a | b | c | d | g; break;
        case '4': digit = b | c | g | f; break;
        case '5': digit = a | c | d | g | f; break;
        case '6': digit = a | g | c | d | e | f; break;
        case '7': digit = a | b | c; break;
        case '8': digit = a | b | c | d | e | f | g; break;
        case '9': digit = a | b | c | d | g | f; break;
        case '-': digit = g; break;
        default: digit = 0; //
    }

    switch (position)
    {
    case 0:
        LCD[2] = ((LCD[2] & 0x1F) | (digit & 0xE0));
        LCD[3] = ((LCD[3] & 0xF0) | (digit & 0x0F));
        break;
    case 1:
        LCD[3] = ((LCD[3] & 0x1F) | (digit & 0xE0));
        LCD[4] = ((LCD[4] & 0xF0) | (digit & 0x0F));
        break;
    case 2:
        LCD[0] = ((LCD[0] & 0x01) | digit);
        break;
    case 3:
        LCD[19] = ((LCD[19] & 0x10) | digit);
        break;
    }
}


/**
 * Ecriture d'un nombre de 4 digits sur l'afficheur haut
 *
 * Entre:
 *   Nombre : nombre de 0  9999
 *   DeuxPoints : vrai, on affiche les deux points. Faux, on les efface
 */
void LCD_4digitafficheurhaut (int nombre, bool_t DeuxPoints)
{
    uint8_t i;

    LCD_effaceafficheurhaut();
    for (i = 0; i < 4; i++)
    {
        if (nombre <= 9999) {
            LCD_writeafficheurhaut (i, (nombre % 10) | 0x30);
            nombre /= 10;
        } else {
            LCD_writeafficheurhaut (i, '-');
        }
    }
    LCD_deuxpoints (0, DeuxPoints);
}

/*
 * Effacement de l'afficheur du haut y compris les points
 */

void LCD_effaceafficheurhaut()
{
    int i;
    for (i = 0; i < 4; i++) {LCD_writeafficheurhaut (i, '\0');}
    LCD_deuxpoints (0, FALSE);
}

